Package: usethis
Title: Automate Package and Project Setup
Version: 3.2.1
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Jennifer", "Bryan", , "jenny@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6983-2759")),
    person("Malcolm", "Barrett", , "malcolmbarrett@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-0299-5825")),
    person("Andy", "Teucher", , "andy.teucher@posit.co", role = "aut",
           comment = c(ORCID = "0000-0002-7840-692X")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Automate package and project setup tasks that are otherwise
    performed manually. This includes setting up unit testing, test
    coverage, continuous integration, Git, 'GitHub', licenses, 'Rcpp',
    'RStudio' projects, and more.
License: MIT + file LICENSE
URL: https://usethis.r-lib.org, https://github.com/r-lib/usethis
BugReports: https://github.com/r-lib/usethis/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.0.1), clipr (>= 0.3.0), crayon, curl (>= 2.7), desc
        (>= 1.4.2), fs (>= 1.3.0), gert (>= 1.4.1), gh (>= 1.2.1), glue
        (>= 1.3.0), jsonlite, lifecycle (>= 1.0.0), purrr, rappdirs,
        rlang (>= 1.1.0), rprojroot (>= 2.1.1), rstudioapi, stats,
        tools, utils, whisker, withr (>= 2.3.0), yaml
Suggests: covr, knitr, magick, pkgload (>= 1.3.2.1), quarto (>= 1.5.1),
        rmarkdown, roxygen2 (>= 7.1.2), spelling (>= 1.2), testthat (>=
        3.1.8)
Config/Needs/website: r-lib/asciicast, tidyverse/tidytemplate, xml2
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Config/testthat/start-first: github-actions, release
Config/usethis/last-upkeep: 2025-04-22
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-09-05 20:27:33 UTC; jenny
Author: Hadley Wickham [aut] (ORCID: <https://orcid.org/0000-0003-4757-117X>),
  Jennifer Bryan [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6983-2759>),
  Malcolm Barrett [aut] (ORCID: <https://orcid.org/0000-0003-0299-5825>),
  Andy Teucher [aut] (ORCID: <https://orcid.org/0000-0002-7840-692X>),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Jennifer Bryan <jenny@posit.co>
Repository: CRAN
Date/Publication: 2025-09-06 05:10:02 UTC
Built: R 4.6.0; ; 2026-04-28 22:39:12 UTC; unix
