---
name: Documentation Task
about: A tightly scoped task for adding or updating documentation.
title: ''
labels: 'documentation'
assignees: nathanjmcdougall

---

**Motivation**
A clear and concise description of what benefit you would get from this documentation update or addition.

**Key Area**
Specify the key area of the project that requires documentation, e.g. the function, the workflow, the developer experience, etc.

**Objectives & Target Audience**
Outline the objectives of the documentation task and describe the target audience. Specify the type of documentation (e.g., API documentation, user guide, tutorial) and what aspect you are specifically trying to communicate.

**Steps**
An outline of the steps required to create or update the documentation. Include details such as:

- The specific content (e.g. text, code snippets, diagrams) to be added or updated.
- The specific affected files
