---
name: Idea
about: A high-level description of a feature or enhancement you'd like.
title: ''
labels: ''
assignees: nathanjmcdougall

---

**Motivation**
A clear and concise description of what benefit you would get from this enhancement.

**Summary of desired enhancement**
A clear and concise description of what you want to happen, at a high level.
