---
name: Feature Development Task
about: A tightly scoped task for a developer.
title: ''
labels: 'enhancement'
assignees: nathanjmcdougall

---

**Motivation**
A clear and concise description of what benefit you would get from this feature.

**Summary of desired feature**
A clear and concise description of what you want to happen.

**Design**
An explanation of the general approach and philosophy of the proposed design.

**Describe alternatives you've considered**
A clear and concise description of any alternative solutions or feature, or designs you've considered.

**Testing Strategy**
What sort of tests will be written?

**Steps**
An outline of steps required to implement this feature.

**Acceptance Criteria**
When would enough be done to consider this task completed?
