---
name: usethis-file-remove
description: Remove files from the project
compatibility: usethis, git
license: MIT
metadata:
  version: "1.0"
---

# Removing Files

## .gitignore

Do not add a file to `.gitignore` simply because it is being deleted. A `.gitignore` entry should only be added when there is a separate, compelling reason — for example, if the file is expected to be regenerated automatically and should not be tracked.
