---
title: "{{{ vignette_title }}}"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{{{ braced_vignette_title }}}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library({{Package}})
```

