# can edit snippets

    Code
      edit_rstudio_snippets("not-existing-type")
    Condition
      Error in `match.arg()`:
      ! 'arg' should be one of "r", "markdown", "c_cpp", "css", "html", "java", "javascript", "python", "sql", "stan", "tex", "yaml"

