# use_tidy_style() is deprecated

    Code
      use_tidy_style()
    Condition
      Warning:
      `use_tidy_style()` was deprecated in usethis 3.2.0.
      i Please use `use_air()` instead.
      i To continue using the styler package, call `styler::style_pkg()` or `styler::style_dir()` directly.

