% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/citation.R
\name{use_citation}
\alias{use_citation}
\title{Create a CITATION template}
\usage{
use_citation()
}
\description{
Use this if you want to encourage users of your package to cite an
article or book.
}
