% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/usethis-defunct.R
\name{usethis-defunct}
\alias{usethis-defunct}
\alias{pr_pull_upstream}
\alias{pr_sync}
\alias{browse_github_token}
\alias{browse_github_pat}
\alias{github_token}
\alias{git_branch_default}
\alias{use_tidy_labels}
\alias{use_tidy_ci}
\alias{use_github_action_check_full}
\title{Defunct and deprecated functions in usethis}
\usage{
pr_pull_upstream()

pr_sync()

browse_github_token(...)

browse_github_pat(...)

github_token()

git_branch_default()

use_tidy_labels()

use_tidy_ci(...)

use_github_action_check_full(
  save_as = "R-CMD-check.yaml",
  ignore = TRUE,
  open = FALSE,
  repo_spec = NULL
)
}
\description{
These functions have either been deprecated or removed from usethis.
}
\section{\code{pr_pull_upstream()}}{

This function has been replaced by \code{\link[=pr_merge_main]{pr_merge_main()}}.
}

\section{\code{pr_sync()}}{

Bundling these operations together did not seem justified, in terms of how
rarely this comes up and, when it does, how likely merge conflicts are.
Users of \code{pr_sync()} should implement these steps "by hand":
\itemize{
\item (Check you are on a PR branch)
\item \code{pr_pull()}
\item \code{pr_merge_main()}, deal with any merge conflicts, if any
\item \code{pr_push()}
}
}

\section{\code{browse_github_token()}, \code{browse_github_pat()}}{

These functions have been replaced by \code{\link[=create_github_token]{create_github_token()}}.
}

\section{\code{github_token()}}{

All implicit and explicit token discovery routes through \code{\link[gh:gh_token]{gh::gh_token()}}
now.
}

\section{\code{git_branch_default()}}{

Please call \code{\link[=git_default_branch]{git_default_branch()}} instead. In hindsight, that is a better
name for this function.
}

\section{\code{use_tidy_labels()}}{

Please call \code{\link[=use_tidy_github_labels]{use_tidy_github_labels()}} instead. In hindsight, that is a
better name for this function.
}

\keyword{internal}
